#include "character.h"
#include <QtXml/qdom.h>
#include <QFile>
#include <QDebug>
#include "interface.h"
#include "game.h"
#include "player.h"

std::vector<CCharacter::SChampDBData>   CCharacter::dataBase;

//------------------------------------------------------
CCharacter::CCharacter()
{
    portrait = -1;
    spell[0] = 0;
    manaRegenTimer.set(MANA_REGEN_TIMER, true);
}

//------------------------------------------------------
bool CCharacter::isDead()
{
    return (stats[eStatHealth].value <= 0);
}

//------------------------------------------------------
// initialize the character with values from the database
void CCharacter::fromDB(int num)
{
    SChampDBData&   db = dataBase[num];

    firstName = db.firstName;
    lastName = db.lastName;
    portrait = db.portrait;

    for (size_t i = 0; i < eStatCount; ++i)
    {
        stats[i].startValue = db.stats[i];
        stats[i].maxValue = db.stats[i];
        stats[i].value = db.stats[i];
    }
    stats[eStatLoad].value = 0;

    for (size_t i = 0; i < eBodyPartCount; ++i)
        bodyObjects[i].setType(0);

    for (size_t i = 0; i < db.objects.size(); ++i)
    {
        int pos = db.objects[i].pos;
        int type = db.objects[i].type;
        bodyObjects[pos].setType(type);
        objects.setDefaultParams(&bodyObjects[pos]);
    }

    updateLoad();
}

//------------------------------------------------------
void CCharacter::readCharactersDB()
{
    // XML tags in the same order as EStatsNames
    static const char   statNames[][20] =
    {
        "health",
        "stamina",
        "mana",
        "strength",
        "dexterity",
        "wisdom",
        "vitality",
        "anti_magic",
        "anti_fire",
        "load"
    };

    QDomDocument doc;

    QFile f("databases/champions.xml");
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomNode    champNode = root.firstChild();

    while(!champNode.isNull())
    {
        if (champNode.isElement())
        {
            QDomElement champion = champNode.toElement();

            if (champion.tagName() == "champion")
            {
                SChampDBData   newChamp;

                std::string firstNameId = champion.attribute("firstName").toUtf8().constData();
                newChamp.firstName = interface.getText(firstNameId);
                std::string lastNameId = champion.attribute("lastName").toUtf8().constData();
                newChamp.lastName = interface.getText(lastNameId);

                QDomElement champInfo = champion.firstChild().toElement();

                while(!champInfo.isNull())
                {
                    if (champInfo.tagName() == "portrait")
                    {
                        newChamp.portrait = champInfo.text().toInt();
                    }
                    else if (champInfo.tagName() == "object")
                    {
                        struct SDBObject    obj;

                        if (champInfo.attribute("pos") == "H")
                            obj.pos = eBodyPartHead;
                        else if (champInfo.attribute("pos") == "N")
                            obj.pos = eBodyPartNeck;
                        else if (champInfo.attribute("pos") == "T")
                            obj.pos = eBodyPartTorso;
                        else if (champInfo.attribute("pos") == "L")
                            obj.pos = eBodyPartLegs;
                        else if (champInfo.attribute("pos") == "F")
                            obj.pos = eBodyPartFeet;
                        else if (champInfo.attribute("pos") == "h")
                            obj.pos = eBodyPartRightHand;
                        else if (champInfo.attribute("pos") == "P")
                            obj.pos = eBodyPartPouch1;
                        else
                            obj.pos = champInfo.attribute("pos").toInt();

                        obj.type = champInfo.text().toInt();
                        newChamp.objects.push_back(obj);
                    }
                    else
                    {
                        for (size_t i = 0; i < eStatCount; ++i)
                        {
                            if (champInfo.tagName() == QString(statNames[i]))
                                newChamp.stats[i] = champInfo.text().toInt();
                        }
                    }
                    champInfo = champInfo.nextSibling().toElement();
                }
                newChamp.stats[eStatLoad] *= 10;
                dataBase.push_back(newChamp);
            }
        }
        champNode = champNode.nextSibling();
    }
}

//------------------------------------------------------
void CCharacter::updateLoad()
{
    int type;
    stats[eStatLoad].value = 0;

    for (int i = 0; i < eBodyPartCount; ++i)
    {
        type = bodyObjects[i].getType();
        stats[eStatLoad].value += objects.mObjectInfos[type].weight;
    }

    if (&game.characters[interface.selectedChampion] == this)
    {
        type = mouse.mObjectInHand.getType();
        stats[eStatLoad].value += objects.mObjectInfos[type].weight;
    }
}

//------------------------------------------------------
int CCharacter::overload()
{
    int value = stats[eStatLoad].value;
    int max = stats[eStatLoad].maxValue;

    if (value > max)
        return 2;
    else if (value > (625 * max) / 1000)
        return 1;
    else
        return 0;
}

//------------------------------------------------------
void CCharacter::load(FILE* handle)
{
    // load the portrait
    fread(&portrait, sizeof(portrait), 1, handle);

    // load the name
    int size;
    char    c;
    fread(&size, sizeof(size), 1, handle);
    firstName.clear();
    for (int i = 0; i < size; ++i)
    {
        fread(&c, 1, 1, handle);
        firstName += c;
    }

    fread(&size, sizeof(size), 1, handle);
    lastName.clear();
    for (int i = 0; i < size; ++i)
    {
        fread(&c, 1, 1, handle);
        lastName += c;
    }

    // load the stats
    for (int i = 0; i < eStatCount; ++i)
    {
        fread(&stats[i].value, sizeof(stats[i].value), 1, handle);
        fread(&stats[i].maxValue, sizeof(stats[i].maxValue), 1, handle);
        fread(&stats[i].startValue, sizeof(stats[i].startValue), 1, handle);
    }

    // load the objects
    for (int i = 0; i < eBodyPartCount; ++i)
        bodyObjects[i].load(handle);

    // load the current spell
    fread(spell, 4, 1, handle);
    spell[4] = 0;
}

//------------------------------------------------------
void CCharacter::save(FILE* handle)
{
    // save the portrait
    fwrite(&portrait, sizeof(portrait), 1, handle);

    // save the name
    int size = firstName.size();
    fwrite(&size, sizeof(size), 1, handle);
    fwrite(firstName.c_str(), size, 1, handle);

    size = lastName.size();
    fwrite(&size, sizeof(size), 1, handle);
    fwrite(lastName.c_str(), size, 1, handle);

    // save the stats
    for (int i = 0; i < eStatCount; ++i)
    {
        fwrite(&stats[i].value, sizeof(stats[i].value), 1, handle);
        fwrite(&stats[i].maxValue, sizeof(stats[i].maxValue), 1, handle);
        fwrite(&stats[i].startValue, sizeof(stats[i].startValue), 1, handle);
    }

    // save the objects
    for (int i = 0; i < eBodyPartCount; ++i)
        bodyObjects[i].save(handle);

    // save the current spell
    fwrite(spell, 4, 1, handle);
}

//------------------------------------------------------
void CCharacter::update()
{
    // update torches
    if (bodyObjects[eBodyPartLeftHand].getType() == OBJECT_TYPE_TORCH)
    {
        CTimer  t;
        t.set(bodyObjects[eBodyPartLeftHand].getIntParam("Charges"), true);
        t.update();
        bodyObjects[eBodyPartLeftHand].setIntParam("Charges", t.get());
    }

    if (bodyObjects[eBodyPartRightHand].getType() == OBJECT_TYPE_TORCH)
    {
        CTimer  t;
        t.set(bodyObjects[eBodyPartRightHand].getIntParam("Charges"), true);
        t.update();
        bodyObjects[eBodyPartRightHand].setIntParam("Charges", t.get());
    }

    // update displayed damages
    damagesTimer.update();

    // mana regeneration
    if (manaRegenTimer.update() == true)
    {
        if (portrait != -1)
            if (stats[eStatMana].value < stats[eStatMana].maxValue)
                stats[eStatMana].value++;

        if (interface.isSleeping() == false)
            manaRegenTimer.set(MANA_REGEN_TIMER, true);
        else
            manaRegenTimer.set(MANA_REGEN_TIMER_SLEEP, true);
    }
}

//------------------------------------------------------
void CCharacter::enterSleep()
{
    manaRegenTimer.set(MANA_REGEN_TIMER_SLEEP, true);
}

//------------------------------------------------------
void CCharacter::exitSleep()
{
    manaRegenTimer.set(MANA_REGEN_TIMER, true);
}

//------------------------------------------------------
int CCharacter::getDexterity()
{
    int dexterity = stats[eStatDexterity].value + RANDOM(8);

    // decrease dexterity depending on the load
    dexterity -= (dexterity / 2) * stats[eStatLoad].value / stats[eStatLoad].maxValue;

    // if we are sleeping...
    if (interface.isSleeping() == true)
        dexterity /= 2;

    // bound the value
    int min = 1 + RANDOM(8);
    int max = 100 - RANDOM(8);

    if (dexterity < min)
        dexterity = min;

    if (dexterity > max)
        dexterity = max;

    return dexterity;
}

//------------------------------------------------------
void CCharacter::hitChampion(int damages)
{
    // TODO: wound, damage types
    displayedDamages = damages;
    damagesTimer.set(90, false);
}
