#include <QDebug>
#include "player.h"
#include "interface.h"
#include "tiles.h"
#include "doors.h"
#include "game.h"

CPlayer  player;

//-----------------------------------------------------------------------------------------
CPlayer::CPlayer()
{
    dir = 0;
    shouldTakeStairs = false;
    shouldTeleport = false;
    shouldFall = false;
}

//-----------------------------------------------------------------------------------------
// transforms a local pos relative to the player into a map pos
CVec2   CPlayer::getPosFromLocal(CVec2 localPos)
{
    CVec2   newLocalPos;

    switch (dir)
    {
        case 0: // up
            newLocalPos = localPos;
            break;

        case 1: // left
            newLocalPos.x = localPos.y;
            newLocalPos.y = -localPos.x;
            break;

        case 2: // down
            newLocalPos.x = -localPos.x;
            newLocalPos.y = -localPos.y;
            break;

        case 3: // right
            newLocalPos.x = -localPos.y;
            newLocalPos.y = localPos.x;
            break;
    }

    return pos + newLocalPos;
}

//-----------------------------------------------------------------------------------------
// This function is the opposite of getPosFromLocal.
// It transforms a map position into a local position relative to the player.
CVec2 CPlayer::getLocalFromPos(CVec2 mapPos)
{
    mapPos -= pos;
    CVec2   newLocalPos;

    switch (dir)
    {
        case 0: // up
            newLocalPos = mapPos;
            break;

        case 1: // left
            newLocalPos.x = -mapPos.y;
            newLocalPos.y = mapPos.x;
            break;

        case 2: // down
            newLocalPos.x = -mapPos.x;
            newLocalPos.y = -mapPos.y;
            break;

        case 3: // right
            newLocalPos.x = mapPos.y;
            newLocalPos.y = -mapPos.x;
            break;
    }

    return newLocalPos;
}

//-----------------------------------------------------------------------------------------
EWallSide   CPlayer::getWallSide(EWallSide side)
{
    static const EWallSide rotatedSides[4][4] =
    {
        {eWallSideUp,    eWallSideDown,  eWallSideLeft,  eWallSideRight}, // up
        {eWallSideLeft,  eWallSideRight, eWallSideDown,  eWallSideUp},    // left
        {eWallSideDown,  eWallSideUp,    eWallSideRight, eWallSideLeft},  // down
        {eWallSideRight, eWallSideLeft,  eWallSideUp,    eWallSideDown}   // right
    };
    return rotatedSides[dir][side];
}

//-----------------------------------------------------------------------------------------
void        CPlayer::turnLeft()
{
    shouldTakeStairs = false;
    shouldTeleport = false;

    if (interface.isInInventory() == false)
        dir = (dir + 1) % 4;

    CTile*  tile = map.getTile(pos);

    if (tile->getType() == eTileStairs)
        shouldTakeStairs = true;
}

//-----------------------------------------------------------------------------------------
void        CPlayer::turnRight()
{
    shouldTakeStairs = false;
    shouldTeleport = false;

    if (interface.isInInventory() == false)
        dir = (dir + 4 - 1) % 4;

    CTile*  tile = map.getTile(pos);

    if (tile->getType() == eTileStairs)
        shouldTakeStairs = true;
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveLeft()
{
    shouldTakeStairs = false;
    shouldTeleport = false;

    if (interface.isInInventory() == false)
    {
        EWallSide   side = getWallSide(eWallSideLeft);
        moveIfPossible(side);
    }
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveRight()
{
    shouldTakeStairs = false;
    shouldTeleport = false;

    if (interface.isInInventory() == false)
    {
        EWallSide   side = getWallSide(eWallSideRight);
        moveIfPossible(side);
    }
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveForward()
{
    shouldTakeStairs = false;
    shouldTeleport = false;

    if (interface.isInInventory() == false)
    {
        EWallSide   side = getWallSide(eWallSideUp);
        moveIfPossible(side);
    }
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveBackward()
{
    shouldTakeStairs = false;
    shouldTeleport = false;

    if (interface.isInInventory() == false)
    {
        CTile*  tile = map.getTile(pos);

        if (tile->getType() == eTileStairs)
        {
            shouldTakeStairs = true;
        }
        else
        {
            EWallSide   side = getWallSide(eWallSideDown);
            moveIfPossible(side);
        }
    }
}

//-----------------------------------------------------------------------------------------
void CPlayer::moveIfPossible(EWallSide side)
{
    CVec2   newPos = pos;

    switch (side)
    {
    case eWallSideUp:
        newPos.y--;
        break;

    case eWallSideDown:
        newPos.y++;
        break;

    case eWallSideLeft:
        newPos.x--;
        break;

    case eWallSideRight:
        newPos.x++;
        break;

    default:
        break;
    }

    CTile*  tile = map.getTile(pos);
    CTile*  destTile = map.getTile(newPos);

    bool canMove = true;

    if (tile->mWalls[side].getType() != 0)
    {
        canMove = false;
    }
    else if (destTile == NULL)
    {
        canMove = false;
    }
    else if (destTile->getType() == eTileDoor && doors.isOpened(destTile) == false)
    {
        canMove = false;
    }

    if (canMove == true)
    {
        pos = newPos;

        if (destTile->getType() == eTileStairs)
            shouldTakeStairs = true;

        if (destTile->getType() == eTileTeleporter && destTile->getBoolParam("forChampions") == true)
            shouldTeleport = true;

        if (destTile->getType() == eTilePit && destTile->getBoolParam("isClosed") == false)
        {
            sound->play(pos, "sound/Scream.wav");
            shouldFall = true;
            interface.setMainState(CInterface::eMainFall);
        }
    }
    else
    {
        if (game.characters[0].portrait != -1)
            sound->play(newPos, "sound/Party_Damaged.wav");
    }
}

//-----------------------------------------------------------------------------------------
void CPlayer::takeStairs()
{
    CTile*  tile = map.getTile(pos);
    int     level = tile->getIntParam("Level");
    CVec2   newPos = CVec2(tile->getIntParam("X"), tile->getIntParam("Y"));
    int     side = tile->getEnumParam("Side");
    game.loadLevel(level, newPos, side);
    shouldTakeStairs = false;
}


//-----------------------------------------------------------------------------------------
void CPlayer::teleport()
{
    CTile*  tile = map.getTile(pos);
    int     level = tile->getIntParam("Level");
    CVec2   newPos = CVec2(tile->getIntParam("X"), tile->getIntParam("Y"));
    int     side = tile->getEnumParam("Side");

    if (side == eWallSideMax)
        side = dir;

    if (level != game.currentLevel)
    {
        game.loadLevel(level, newPos, side);
    }
    else
    {
        pos = newPos;
        dir = side;
    }
    sound->play(pos, "sound/Buzz.wav");
    shouldTeleport = false;
}

//-----------------------------------------------------------------------------------------
void CPlayer::fall()
{
    CTile*  tile = map.getTile(pos);
    int     level = tile->getIntParam("Level");
    CVec2   newPos = CVec2(tile->getIntParam("X"), tile->getIntParam("Y"));

    game.loadLevel(level, newPos, dir);
    shouldFall = false;
    interface.setMainState(CInterface::eMainGame);
}
